	SUBROUTINE GEODCGEOMAG(DAY,UT,LAT,LON,MLAT,MLON,MLT,LER)

!      PURPOSE
!     To calculate the corrected geomagnetic latitude and local time from
!     geocentric coordinates.
!
!  METHOD
!     No discussion
!
!  INPUT PARAMETERS
!     DAY     INTEGER           The day of the year
!     UT      REAL              The universal time, in seconds
!     LAT     REAL              The geocentric latitude, in degrees north
!     LON     REAL              The geocentric longitude, in degrees east
!
!  OUTPUT PARAMETERS
!     MLAT    REAL              The corrected geomagnetic latitude, in degrees north
!     MLON    REAL              The corrected geomagnetic longitude, in degrees east
!     MLT     REAL              The corrected geomagnetic local time, in hours
!
!  LOCAL VARIABLES
!     LATAS   REAL              The geocentric latitude of the antisolar point, in degrees north
!     LONAS   REAL              The geocentric longitude of the antisolar point, in degrees east
!     MLATAS  REAL              The corrected geomagnetic latitude of the antisolar point, in
!                               degrees north
!     MLONAS  REAL              The corrected geomagnetic longitude of the antisolar point, in
!                               degrees east
!
      
        DLL_EXPORT GEODCGEOMAG
 
!	IMPLICIT NONE

	CHARACTER*50 PATH
	INTEGER LUN 
	REAL UT,LAT,LON,MLAT,MLON,MLT
	integer DAY,LER,I,J,K

       INCLUDE 'cgdb.inc'

      
!	DAY = 231
!	UT = 0
!	LAT = -22
!	LON = -51
!
!	LUN =1
      IF (LER.EQ.1) THEN
!	  PATH = '!:\gpseq_BC_Ha\campo_geomagnetico\PIM\CGMDB\'

  	  CALL RDCGDB

	ELSE 
!	  OPEN(UNIT=2,FILE='CG.TXT',STATUS='UNKNOWN',FORM='FORMATTED')
!	 WRITE(2,100) (((CG(I,J,K),I=1,2),J=1,36),K=1,89)
!  100 FORMAT(0P8F6.1)

	ENDIF
	

!	DO LAT=-88,88,2
      
!	  DO LON=-175,175,5

	    CALL GCXCGM(DAY,UT,LAT,LON,MLAT,MLON,MLT)
!          WRITE(*,*)DAY,UT,LAT,LON,MLAT,MLON
!		LON = LON + 5
		 
!	  ENDDO
!	LAT=LAT+2
!      ENDDO


	END



      SUBROUTINE RDCGDB
!  PURPOSE
!  Subroutine to read the CG DataBase
!
!  METHOD
!     No discussion
!
!  INPUT PARAMETERS
!    NAME    TYPE       ARRAY  description
!    LUN     INTEGER           Logical Unit Number of the CGM database
!    PATH    CHARACTER         Path to the CGM database
!
!  OUTPUT PARAMETERS
!    NAME    TYPE       ARRAY  description
!    CG      REAL    (2,36,89) The array holding the coefficients from
!                              geographic to corrected geomagnetic coordinates.
!
!  LOCAL VARIABLES
!    NAME    TYPE       ARRAY  description
!    FSPEC   CHAR*100          The file specification of the CGM database
!    I       INTEGER           Do Loop Index
!    J       INTEGER           Do Loop Index
!    K       INTEGER           Do Loop Index
!
!  SUBROUTINES CALLED
!     STRCCT   Concatenates two character strings
!
!  FUNCTIONS CALLED
!    NONE
!
!  FILES ACCESSED
!     NAME                   description
!     cglalo.dat             Holds the CG array
!
!  AUTHOR
!     William G. Whartenby
!     Computational Physics, Inc.
!     240 Bear Hill Road  Suite 202A
!     Waltham, MA 02154  USA
!     (617)-487-2250
!
!  VERSION
!     1.3   25-September-1995
!
!  MODIFICATIONS
!     ----Person----  ----Date----  ----------------Description----------------
!     W. Whartenby    21-Dec-1990   1.0.6
!     L. Brown        25-Sep-1995   1.0.6 ==> 1.3
!                                   The concatenation of the path and file name
!                                   has been replaced by a call to routine
!                                   STRCCT in order to resolve an
!                                   incompatibility with the Watcom and
!                                   Microsoft FORTRAN compilers.  This problem
!                                   was reported by Jerry Ferguson and George
!                                   Bendo.
!
!  REFERENCES
!     NONE
!
!  SPECIAL CONSTANTS
!     NONE
!
      INCLUDE 'cgdb.inc'

      
      CHARACTER*50 PATH
      CHARACTER*100 FSPEC
      INTEGER I,J,K,LUN
!
       
       LUN = 1
!      CALL STRCCT(PATH,'cglalo.dat',32,FSPEC,I)

      OPEN(UNIT=LUN,FILE='CGLALO.DAT',STATUS='OLD',FORM='FORMATTED',ERR=10)
      
	READ(LUN,100) (((CG(I,J,K),I=1,2),J=1,36),K=1,89)
      CLOSE(UNIT=LUN)
!


      RETURN
  100 FORMAT(0P8F6.1)
  10	write(*,*) 'Error opening CGLATLO file in cgm_util.f'
      write(*,*)' '
!	pause
	stop
      END
!
      SUBROUTINE CGINV1(CGLAT,CGLON,GGLAT,GGLON)
!
!  PURPOSE
!    This subroutine converts coorected geomagnetic
!  coordinates back to geographic coordinates, be-
!  cause some of the HLISM model applications soft-
!  ware requires the coordinates in that frame. The
!  subroutine was developed by Logicon,Inc. in July
!  1973 and later modified by Ms. Hausman of NOAA/NGDC,
!  Boulder,CO and Rod Cregier, USAF Academy, CO. CGINV
!  is the driver of the conversion, and calls CGLAL1.
!
!  METHOD
!    1. Selects polar or mid-latitude algorithms
!    2. If polar gridpoint, converts the gridpoint
!       using polar stereographic coordinates to
!       geographic
!    3. If mid-latitude gridpoint, converts the grid-
!       point using rectangular coordinates to
!       geographic.
!
!  INPUT PARAMETERS
!    NAME    TYPE       ARRAY  description
!    CGLAT   REAL              Corrected Geomagnetic Latitude
!    CGLON   REAL              Corrected Geomagnetic Longitude
!
!  OUTPUT PARAMETERS
!    NAME    TYPE       ARRAY  description
!    GGLAT   REAL              Geographic Latitude
!    GGLON   REAL              Geographic Longitude
!
!  LOCAL VARIABLES
!    NAME    TYPE       ARRAY  description
!    N       INTEGER           North/south flag (n=1,s=2)
!    BA      REAL              Difference between gridpoint long-
!                              itude and the geomag longitude of
!                              pole in radians
!    BL      REAL              Geomag longitude of pole in radians
!    CGLAT   REAL              Input geomag latitude
!    CGLON   REAL              Input geomag longitude
!    ELAT    REAL              Incremental difference in latitude
!    ELON    REAL              Incremental difference in longitude
!    FRACT   REAL
!    GGLAT   REAL              Returned geographic latitude
!    GGLON   REAL              Returned geographic longitude
!    I       INTEGER           Do loop index
!    PFLAG   LOGICAL           .TRUE. when input is near magnetic pole
!    PLAT    REAL       (2)    Geographic latitude of poles
!    PLONG   REAL       (2)    Geographic longitude of poles
!    PMAT    REAL       (2)    Geomagnetic latitude of poles
!    PMONG   REAL       (2)    Geomagnetic longitude of poles
!    QLAT    REAL              Temporary latitude increment value
!    QLONG   REAL              Temporary longitude increment value
!    RAD     REAL              Radian to degree conversion factor
!    RLAT    REAL       (2)    Ratio of differences between geogra-
!                              PHIC and geomagnetic latitude at pole
!    RLONG   REAL       (2)    Ratio of defference between geograp-
!                              hic and geomagnetic longitude at pole
!    SB      REAL              Ratio of (90-lat) to (90-pole)
!    SC      REAL              (90-pole) in geomagnetic coordinates
!    SGN     REAL              Sign of latitude (n=1,s=-1)
!    SVCGLT  REAL              Temporary variable to save original
!                              value of cglat when near mag pole
!    X       REAL              First guess geographic latitude
!    Y       REAL              First guess geographic longitude
!
!   SUBROUTINES CALLED
!    NAME     description
!    CGLAL1   Converts geographic to geomagnetic
!
!  FUNCTIONS CALLED
!    NONE
!
!  FILES ACCESSED
!     NONE
!
!  AUTHOR
!  JAN 86 - NATIONAL GEOPHYSICAL DATA CENTER
!
!  VERSION
!     1.0.6   22-Dec-1990
!
!  MODIFICATIONS
!     ----Person----  ----Date----  ----------------Description----------------
!     AFGWC/SDDE       May 86        Upgraded documentation
!     NATIONAL GEO-    Aug 87
!     PHYSICAL DATA
!     CENTER (87-I)
!     NGDC (88-II)     Mar 88        Replace CORGM1 with CGLAL1
!     CPI              Mar 89        added special case in which input is
!                                    is either magnetic pole; also added
!                                    coding to take care of cases within a
!                                    degree of magnetic poles
!     CPI              Jan 91        Changed the header and declarations to
!                                    current format
!
!  REFERENCES
!  1.  ICED-III SYSTEM DOCUMENTATION
!  2.  SPERRY FORTRAN-77 (ASCII) PRM
!  3.  AFGWC FORM 10 DX-50194
!  4.  AFGWC FORM 10 DX-60096
!
!  SPECIAL CONSTANTS
!     NONE
!
      INTEGER N,I
      LOGICAL PFLAG
      REAL BA,BL,CGLAT,CGLON,ELAT,ELONG,GGLAT,GGLON,PLAT(2),PLONG(2)
      REAL PMAT(2),PMONG(2),QLAT,QLONG,RAD,RLAT(2),RLONG(2),SB,SC,SGN
      REAL SVCGLT,X,Y,FRACT
!
      DATA RAD/57.2957795/
      DATA PLAT/80.,75./,PLONG/280.,127./
      DATA PMAT/81.6,74.5/,PMONG/169.8,17.3/
!
!
!     If input is at either pole, bypass search
!
      IF (CGLAT .EQ. 90.0) THEN
        GGLAT = PLAT(1)
        GGLON = PLONG(1)
        RETURN
      ELSE IF (CGLAT .EQ. -90.0) THEN
        GGLAT = -PLAT(2)
        GGLON = PLONG(2)
        RETURN
      ENDIF
!
!     Begin execution by choosing polar or mid-latitude
!
      IF(ABS(CGLAT) .GE. 40.) THEN
!
!      Polar case -- search using polar coordinates.
!       First check to see if input coordinate is within one degree
!       pole.  If so, calculate coordinates of point at same CGLON.
!       But at CGLAT = 89.0, then linearly interpolate
!
       IF (CGLAT .GT. 89.0) THEN
        SVCGLT = CGLAT
        CGLAT = 89.0
        PFLAG = .TRUE.
       ELSE IF (CGLAT .LT. -88.0) THEN
        SVCGLT = CGLAT
        CGLAT = -88.0
        PFLAG = .TRUE.
       ELSE
        PFLAG = .FALSE.
       ENDIF
!
       DO I = 1,2
        RLONG(I)=PLONG(I)+180.-PMONG(I)
        RLAT(I)=(90.-PMAT(I))/(90.-PLAT(I))
       ENDDO
       N=1
       IF(CGLAT.LT.0.) N=2
       SGN=3.-2.*N
       SC=90.-PLAT(N)
       SB=(90.-ABS(CGLAT))/RLAT(N)
       BA=(CGLON+RLONG(N))/RAD
       BL=PLONG(N)/RAD
       DO I = 1,50
        X=SC*COS(BL)+SB*COS(BA)
        Y=SC*SIN(BL)+SB*SIN(BA)
        GGLAT=(90.-SQRT(X*X+Y*Y))*SGN
        GGLON=ATAN2(Y,X)*RAD
        IF(GGLON.LT.0.) GGLON=GGLON+360.
        CALL CGLAL1(GGLAT,GGLON,QLAT,QLONG)
        ELAT=(CGLAT-QLAT)*SGN
        ELONG=CGLON-QLONG
        IF(ABS( ELAT).GT.0.05.AND.ABS( ELAT).LT.359.95) GO TO 120
        IF(ABS(ELONG).GT.0.05.AND.ABS(ELONG).LT.359.95) GO TO 120
        GO TO 135
120     SB=SB-ELAT
        BA=BA+ELONG/RAD
       ENDDO
135    CONTINUE
       IF (PFLAG) THEN
        IF (N .EQ. 1) THEN
          FRACT = SVCGLT - CGLAT
        ELSE
          FRACT = 0.5*(CGLAT - SVCGLT)
        ENDIF
        GGLAT = GGLAT + FRACT*(SGN*PLAT(N) - GGLAT)
        GGLON = GGLON + FRACT*(PLONG(N) - GGLON)
        CGLAT = SVCGLT
       ENDIF
      ELSE
!
!      Mid-latitude case -- search using rectangular coordinates.
!
       GGLAT=CGLAT
       GGLON=AMOD(CGLON+290.,360.)
       DO I = 1,20
        CALL CGLAL1(GGLAT,GGLON,QLAT,QLONG)
        ELAT=CGLAT-QLAT
        ELONG=CGLON-QLONG
        IF(ABS( ELAT).GT.0.05.AND.ABS( ELAT).LT.359.95) GO TO 150
        IF(ABS(ELONG).GT.0.05.AND.ABS(ELONG).LT.359.95) GO TO 150
        RETURN
150     GGLAT=GGLAT+ELAT
        GGLON=AMOD(GGLON+ELONG+360.,360.)
       ENDDO
      ENDIF
      RETURN
      END
!
      SUBROUTINE CGLAL1( GGLAT,GGLON, CGLAT,CGLON )
!
!  PURPOSE
!    This subroutine is the Hakura-Gustofsson coord-
!  inate transformation of geographics lat and long
!  to corrected geomagnetic lat and long.  the
!  routine has been changed many times.
!
!
!  METHOD
!  1.  Insure lat & lon are within proper limits
!  2.  Find the four corners of a rectangle con-
!      taining the requested gridpoint.
!  3.  Check the north, south, east and west bound-
!      aries of the box to insure they are within
!      the transformation array.
!  4.  Convert the gridpoint to geomag lat and lon.
!
!  INPUT PARAMETERS
!    NAME    TYPE       ARRAY  description
!    GGLAT   REAL              Geographic Latitude
!    GGLON   REAL              Geographic Longitude
!
!  OUTPUT PARAMETERS
!    NAME    TYPE       ARRAY  description
!    CGLAT   REAL              Corrected Geomagnetic Latitude
!    CGLON   REAL              Corrected Geomagnetic Longitude
!
!  LOCAL VARIABLES
!    NAME    TYPE       ARRAY  description
!    I       INTEGER           Quality control variable
!    J       INTEGER           Index into longitude / 10
!    K       INTEGER           Index into data array
!    ALPHA   REAL              Coefficient of interpolation
!    BETA    REAL              Coefficient of interpolation
!    CG      REAL    (2,36,89) Geomagnetic coordinates of gridpoints
!    CGLAT   REAL              Corrected geomag latitude
!    CGLON   REAL              Corrected geomag longitude
!    CGMNLA  REAL              Mag latitude for north pole
!    CGMNLO  REAL              Mag longitude for north pole
!    CGMSLA  REAL              Mag latitude for south pole
!    CGMSLO  REAL              Mag longitude for south pole
!    EGGLA   REAL              Southern geographic latitude
!    GGLAT   REAL              Geographic latitude
!    GGLAT1  REAL              Temp. geographic latitude
!    GGLON   REAL              Geographic longitude
!    GGLON1  REAL              Temp. geographic latitude
!    TEMP    REAL              Local calculation variable
!    WGGLO   REAL              Western boundary of grid
!    X1LAT   REAL              Northeast latitude of box
!    X1LON   REAL              Northeast longitude of box
!    X2LAT   REAL              Northwest latitude of box
!    X2LON   REAL              Northwest longitude of box
!    X3LAT   REAL              Southwest latitude of box
!    X3LON   REAL              Southwest longitude of box
!    X4LAT   REAL              Southeast latitude of box
!    X4LON   REAL              Southeast longitude of box
!
!   SUBROUTINES CALLED
!    NONE
!
!  FUNCTIONS CALLED
!    NONE
!
!  FILES ACCESSED
!    NONE
!
!  AUTHOR
!  JAN 86 - NATIONAL GEOPHYSICAL DATA CENTER
!
!  VERSION
!     1.0.6   22-Dec-1990
!
!  MODIFICATIONS
!     ----Person----  ----Date----  ----------------Description----------------
!     R.K. ROSICH     23 Feb 79     Corrected to allow calculation of
!                                   CGM up to both poles
!     F.J.RICH        AUG 83        Modified to use arguments based on
!                                   IGRF80 and discontinuities in table
!                                   removed by curve fitting.
!                     JUL 85        Corrected and updated by Herbert
!                                   Kroehl and Bonnie Hausman of
!                                   USDOC/NOAA/NGDC/DSD
!     AFFGWC/SDDE     MAY 86        Upgraded documentation
!     NGDC (87-I)     AUG 87
!     NGDC (88-II)    MAR 88        Documentation requirements
!     CPI             MAR 89        Renamed COMMON/COORDS/ to COMMON/CGDB/
!                                   and corrected coordinates to be con-
!                                   sistent with CGINV1
!     CPI             MAR 90        Changed the COMMON/CGDB/ to an in-
!                                   clude statement
!     CPI             Jan 91        Changed the header,declarations and
!                                   comments to current format
!
!  REFERENCES
!  1.  ICED-III SYSTEM DOCUMENTATION
!  2.  SPERRY FORTRAN-77 (ASCII) PRM
!  3.  AFGWC FORM 10 DX-50194
!  4.  AFGWC FORM 10 DX-60096
!
!  SPECIAL CONSTANTS
!     NONE
!
      INTEGER I,J,K
      REAL ALPHA,BETA,CGLAT,CGLON,CGMNLA,CGMNLO,CGMSLA,CGMSLO,EGGLA
      REAL GGLAT,GGLAT1,GGLON,GGLON1,TEMP,WGGLO,X1LAT,X1LON,X2LAT
      REAL X2LON,X3LAT,X3LON,X4LAT,X4LON,Y
!
      INCLUDE 'cgdb.inc'
      
!
!  Commented out data statement is from MAR 88 version received
!    from NGDC.  The values in current use are consistent with
!    CGINV1    ---RED 3/2/89
!     DATA CGMNLA,CGMNLO/82.1,170.8/,  CGMSLA,CGMSLO/-74.0,19.5/
!
      DATA CGMNLA,CGMNLO/81.6,169.8/,  CGMSLA,CGMSLO/-74.5,17.3/
!
!     Save original inputs
!
      GGLAT1 = GGLAT
      GGLON1 = GGLON
!
!     Reduce GGLAT To the range -90 to +90 degrees
!
      DO WHILE (GGLAT1 .GT. 90.)
       GGLAT1=180.-GGLAT1
       GGLON1=GGLON1+180.
      ENDDO
      DO WHILE (GGLAT1 .LT. -90.)
       GGLAT1=-180.-GGLAT1
       GGLON1=GGLON1+180.
      ENDDO
!
!     Reduce GGLON to .lt. 360 degrees (positive or negative)
!
 40   GGLON1 = AMOD(GGLON1,360.)
!
!     Check to see if GGLON > 0, if not correct to be > zero
!
      IF(GGLON1 .LT. 0.0)GGLON1 = GGLON1 + 360.0
      I = IFIX((90.0 - GGLAT1)/2.0)
      J = IFIX(GGLON1/10.) + 1
!
!     Check to see if I is within bounds
!
      IF((I .LE. 0) .OR. (I.GE.89)) THEN
!
!      Set to interpolate to the poles (N or S geographic)
!
       IF (J .GT. 36) J=J-36
       IF (J .LT.  1) J=J+36
       K=J+1
       IF (K .GT. 36) K=K-36
       IF (I .LE.  0) THEN
        I=0
        X1LAT=CGMNLA
        X1LON=CGMNLO
        X2LAT=CG(1,J,I+1)
        X2LON=CG(2,J,I+1)
        X3LAT=CG(1,K,I+1)
        X3LON=CG(2,K,I+1)
        X4LAT=CGMNLA
        X4LON=CGMNLO
       ELSE
        I=89
        X1LAT=CG(1,J,I)
        X1LON=CG(2,J,I)
        X2LAT=CGMSLA
        X2LON=CGMSLO
        X3LAT=CGMSLA
        X3LON=CGMSLO
        X4LAT=CG(1,K,I)
        X4LON=CG(2,K,I)
       ENDIF
      ELSE
!
!      Statement 60.. check to see if J is within bounds.  If out of
!      bounds, restore to in bounds.
!
       IF(J.GT.36)J = J - 36
       IF(J.LT.1)J = J + 36
!
!      Find CGLAT and CGLON of the four corners of rectangle within
!      which GGLAT and GGLON fall.  The corners of the rectangle are X1,
!      X2,X3 and X4.  X1 refers to northwest corner of the rectangle.
!      The labeling goes in a counterclockwise manner.
!
       X1LAT = CG(1,J,I)
       X1LON = CG(2,J,I)
       X2LAT = CG(1,J,I+1)
       X2LON = CG(2,J,I+1)
!
!      Check to see if the easterly edge of the rectangle is within the
!      limits of the argument bounds of array CG
!
       K = J + 1
       IF(K .GT. 36)K = K - 36
       X3LAT = CG(1,K,I+1)
       X3LON = CG(2,K,I+1)
       X4LAT = CG(1,K,I)
       X4LON = CG(2,K,I)
      ENDIF
!
!      Eliminate the discontinuities in the X1, X2, X3, and X4 lon which
!      would occur if the prime meridian of the corrected geomagnetic
!      coordinate system passed through rectangle X1,X2,X3,X4.
!
      Y = AMAX1(X1LON,X2LON,X3LON,X4LON)
      IF(ABS(Y-X1LON).GT.180.0)X1LON = X1LON + 360.0
      IF(ABS(Y-X2LON).GT.180.0)X2LON = X2LON + 360.0
      IF(ABS(Y-X3LON).GT.180.0)X3LON = X3LON + 360.0
      IF(ABS(Y-X4LON).GT.180.0)X4LON = X4LON + 360.0
!
!     Calculate the southern and the westward boundaries (geographic
!     latitude and longitude) of the rectangle.
!
      EGGLA = FLOAT(-2*I + 88)
      WGGLO = FLOAT(10*J - 10)
!
!     Calculate the coefficients of the interpolation equation.  These
!     coefficients represent the fractional part of the box the point
!     GGLAT-GGLON is from the southern and the westward boundaries of
!     the rectangle.
!
      ALPHA = (GGLAT1 - EGGLA)/2.0
      BETA = (GGLON1 - WGGLO)/10.
!
!     Now calculate the corrected geomagnetic coordinates that corresp-
!     pond to geographic coordinates of the input point (GGLAT,GGLON)
!
      TEMP = (ALPHA*BETA)*(X4LAT+X2LAT-X1LAT-X3LAT)
      CGLAT = X2LAT + ALPHA*(X1LAT-X2LAT) + BETA*(X3LAT-X2LAT) + TEMP
      TEMP = (ALPHA*BETA)*(X4LON+X2LON-X1LON-X3LON)
      CGLON = X2LON + ALPHA*(X1LON-X2LON) + BETA*(X3LON-X2LON) + TEMP
!
!     Check to determine if the corrected geomagnetic longitude is less
!     than 360 degrees.  If it is not, reduce the calculated value by
!     360 degrees.
!
      IF(CGLON.GT.360.0)CGLON = CGLON - 360.0
      RETURN
      END
!
      FUNCTION FAP(KP)
!
!  PURPOSE
!  Calculates the quasi-linear magnetic activity index Ap from the
!  quasi-logarithmic index Kp using Table 6 from Mayaud (1980).
!
!  METHOD
!   Table lookup in which the subscript is just 3*Kp, followed by
!   linear interpolation (to allow for averaged Kp's)
!
!  INPUT PARAMETERS
!    NAME    TYPE       ARRAY  description
!    KP      REAL            The magnetic activity index Kp.
!
!  OUTPUT PARAMETERS
!    NAME    TYPE       ARRAY  description
!    FAP     REAL            The magnetic activity index Ap.
!
!  LOCAL VARIABLES
!    NAME    TYPE       ARRAY  description
!    IKP     INTEGER           Integer of TKP
!    TKP     REAL              3*KP
!
!  SUBROUTINES REQUIRED
!     None.
!
!  FILES ACCESSED
!     None.
!
!  AUTHOR
!     Robert E. Daniell
!     Computational Physics, Inc.
!     240 Bear Hill Road  Suite 202A
!     Waltham, MA 02154  USA
!     (617)-487-2250
!
!  VERSION
!     1.0.6    22-June-1990
!
!  MODIFICATIONS
!     ----Person----  ----Date----  ----------------Description----------------
!
!  REFERENCES
!      Mayaud, P. N., Derivation, Meaning, and Use of Geomagnetic
!        Indices, Geophysical Monograph 22, American Geophysical
!        Union, Washington, DC, 1980
!
!  SPECIAL CONSTANTS
!     NONE
!
      REAL KP,TKP,FAP,AP1,AP2
      INTEGER KAP(0:27),IKP
!     +  +         +         +         +         +         +         + +
!  3*Kp:         0,  1,  2,  3,  4,  5,  6,  7,  8,  9, 10, 11, 12, 13,
!               14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27/
      DATA KAP/  0,  2,  3,  4,  5,  6,  7,  9, 12, 15, 18, 22, 27, 32, &
               39, 48, 56, 67, 80, 94,111,132,154,179,207,236,300,400/
!
      IF (KP .LT. 0.0) KP = 0.0
      IF (KP .GT. 9.0) KP = 9.0
!
      TKP = 3.0*KP
      IKP = IFIX(TKP)
      IF (IKP .EQ. 27) IKP = IKP - 1
      AP1 = FLOAT(KAP(IKP))
      AP2 = FLOAT(KAP(IKP+1))
      FAP = AP1 + (AP2-AP1)*(TKP - FLOAT(IKP))
!
      RETURN
      END
!
      SUBROUTINE GCXCGM(DAY,UT,LAT,LON,MLAT,MLON,MLT)
!
!  PURPOSE
!     To calculate the corrected geomagnetic latitude and local time from
!     geocentric coordinates.
!
!  METHOD
!     No discussion
!
!  INPUT PARAMETERS
!     DAY     INTEGER           The day of the year
!     UT      REAL              The universal time, in seconds
!     LAT     REAL              The geocentric latitude, in degrees north
!     LON     REAL              The geocentric longitude, in degrees east
!
!  OUTPUT PARAMETERS
!     MLAT    REAL              The corrected geomagnetic latitude, in degrees north
!     MLON    REAL              The corrected geomagnetic longitude, in degrees east
!     MLT     REAL              The corrected geomagnetic local time, in hours
!
!  LOCAL VARIABLES
!     LATAS   REAL              The geocentric latitude of the antisolar point, in degrees north
!     LONAS   REAL              The geocentric longitude of the antisolar point, in degrees east
!     MLATAS  REAL              The corrected geomagnetic latitude of the antisolar point, in
!                               degrees north
!     MLONAS  REAL              The corrected geomagnetic longitude of the antisolar point, in
!                               degrees east
!
!  SUBROUTINES REQUIRED
!     CGLAL1   Converts geocentric latitude and longitude to corrected geo-
!              magnetic latitude and longitude
!
!  FILES ACCESSED
!     NONE
!
!  AUTHOR
!     Lincoln Brown
!     Computational Physics, Inc.
!     240 Bear Hill Road  Suite 202A
!     Waltham, MA 02154  USA
!     (617)-487-2250
!
!  VERSION
!     1.0.6      22-June-1990
!
!  MODIFICATIONS
!     ----Person----  ----Date----  ----------------Description----------------
!
!  REFERENCES
!     NONE
!
!  SPECIAL CONSTANTS
!     NONE
!
      INTEGER DAY
      REAL UT,LAT,LON,MLAT,MLT
      REAL MLON,LATAS,LONAS,MLATAS,MLONAS
!
!  Calculate the geomagnetic coordinates of the geocentric location
!
      CALL CGLAL1(LAT,LON,MLAT,MLON)

!
!  Calculate the geocentric coordinates of the antisolar point
!
      LATAS=23.45*COS(6.283185*FLOAT(DAY+10)/365.0)
      LONAS=360.-UT/240.
!
!  Calculate the geomagnetic coordinates of the antisolar point
!
      CALL CGLAL1(LATAS,LONAS,MLATAS,MLONAS)
!
!  Calculate the magnetic local time
!
      MLT=MOD(24.+(MLON-MLONAS)/15.,24.)
!
      RETURN
      END
!
      SUBROUTINE BNDRY(CGMLAT,CGMLT,RB)
!
!
!  PURPOSE
!     To determine the high latitude region of a given location.
!
!  METHOD
!     Three regions are defined for the high latitude ionosphere:  polar cap,
!     auroral oval, and trough.  The equatorward boundary of each region is
!     represented by a circle with centers given in the common block PRECIP.
!     See subroutine INIT for a description of how these values are determined.
!
!  INPUT PARAMETERS
!    LAT0    REAL       (3)    The Geomagnetic latitude corresponding
!                              to the circle center of the oval region
!                       (i)
!                        i  =  1) Equatorward value
!                              2) Value of maximum
!                              3) Poleward value
!    MLT     REAL              The corrected geomagnetic local time, in hours
!    MLT0    REAL       (3)    The Geomagnetic local time corresponding
!                              to the circle centero f the oval region
!                       (i)
!                        i  =  1) Equatorward value
!                              2) Value of maximum
!                              3) Poleward value
!    RAD0    REAL       (3)    The radius of the circle of the oval region
!                       (i)
!                        i  =  1) Equatorward value
!                              2) Value of maximum
!                              3) Poleward value
!    LATR    REAL              The Geomagnetic latitude corresponding
!                              to the circle center of the trough region
!    MLTR    REAL              The Geomagnetic local time corresponding
!                              to the circle centero f the trough region
!    RADTR   REAL              The radius of the circle of the trough region
!
!
!  OUTPUT PARAMETERS
!     RB      REAL       ARRAY  Gives the equatorward boundaries of the three
!                                 regions
!  LOCAL VARIABLES
!     I       INTEGER           1 for northern hemsiphere, 2 otherwise
!
!  SUBROUTINES REQUIRED
!     NONE
!
!  FILES ACCESSED
!     NONE
!
!  AUTHOR
!     Robert E. Daniell
!     Computational Physics, Inc.
!     240 Bear Hill Road  Suite 202A
!     Waltham, MA 02154  USA
!     (617)-487-2250
!
!  VERSION
!     1.0.6    22-June-1990
!
!  MODIFICATIONS
!     ----Person----  ----Date----  ----------------Description----------------
!
!  REFERENCES
!     NONE
!
!
!
!      INCLUDE 'indirect.inc'
!    Variables in common block INDIRECT
!    NAME      TYPE ARRAY  description
!    AP        REAL        Computed Ap from user supplied E layer Kp.
!    BY        REAL        Y component of the IMF
!    BZ        REAL        Z component of the IMF
!    EKP       REAL        E Kp for choosing the USU database, or to
!                          iterate on in HLEMODEL
!    ESWITCH   LOGICAL     .TRUE. if the CCIR foE model is to be used for foE,
!                          .FALSE. for parameterized model foE
!    F10P7     REAL        F10P7 to iterate on in HLEMODEL
!    KP        REAL        F Kp
!    RF10P7    REAL        Kp read from the station files for calls to NEUATM
!    RKP       REAL        Kp read from the station files for computing AP
!    URSISW    LOGICAL     .TRUE. if use URSI coefficients for FoF2 normalization,
!                          .FALSE. if the parameterized model used
!
       REAL KP(0:2),AP,BY,BZ
       REAL F10P7,EKP
       REAL RKP,RF10P7
       LOGICAL URSISW,ESWITCH
!
       COMMON/INDIRECT/EKP,KP,F10P7,BY,BZ,AP,&
           RF10P7,RKP,URSISW,ESWITCH
!

      INCLUDE 'precip.inc'
      INCLUDE 'aindex.inc'
      INCLUDE 'region_b.inc'
      REAL LAT1(3),MLT1(3),RAD1(3)
      REAL RB(0:2),CGMLT,CGMLAT
      INTEGER I
!
!     Set up the latitude-local time- radius array to correspond
!     to the trough-oval-cap boundary
      I = 1
      IF (CGMLAT .LT. 0.0) I = 2
      LAT1(1) = MLATR0(I)
      MLT1(1) = MLTR0(I)
      RAD1(1) = ATR(0,1) +ATR(1,1)*KP(0)
      LAT1(2) = LAT0(1,1,1,I)
      MLT1(2) = MLT0(1,1,1,I)
      RAD1(2) = A(0,1,1)+A(1,1,1)*KP(1)
      LAT1(3) = LAT0(3,1,1,I)
      MLT1(3) = MLT0(3,1,1,I)
      RAD1(3) = A(0,2,1)+A(1,2,1)*KP(2)
      CALL PREGION(CGMLT,LAT1,MLT1,RAD1,RB)
!     Make sure minimum widths are met
      IF ((RB(1) -RB(0)) .LT. CB(1)) THEN
       RB(0) = RB(1) - CB(1)
      ENDIF
      IF ((RB(2) -RB(1)) .LT. CB(2)) THEN
       RB(2) = RB(1) + CB(2)
      ENDIF
      RETURN
      END
!
      FUNCTION KPTOAP(KP)
!
!  PURPOSE
!     Calculates the quasi-linear magnetic activity index Ap from the
!     quasi-logarithmic index Kp using Table 6 from Mayaud (1980).
!
!  METHOD
!     Table lookup in which the subscript is just Kp*3, followed by
!     linear interpolation (to allow for averaged Kp's).
!
!  INPUT PARAMETERS
!     KP       The magnetic activity index Kp
!
!  OUTPUT PARAMETERS
!     KPTOAP   The magnetic activity index Ap
!
!  LOCAL VARIABLES
!    IKP3      KP3 truncated to an integer
!    KAP       Ap corresponding to Kp*3
!    KP3       KP*3
!
!  SUBROUTINES REQUIRED
!     NONE
!
!  FILES ACCESSED
!     NONE
!
!  AUTHOR
!     Robert E. Daniell
!     Computational Physics, Inc.
!     240 Bear Hill Road  Suite 202A
!     Waltham, MA 02154  USA
!     (617)-487-2250
!
!  VERSION
!     1.0.6    12-November-1991
!
!  MODIFICATIONS
!     ----Person----  ----Date----  ----------------Description----------------
!     R. Daniell      22-Jun-1990   1.0
!                                   Created
!     L. Brown        12-Nov-1991   1.0 --> 1.1
!                                   Cleaned up code.
!
!  REFERENCES
!      Mayaud, P. N., Derivation, 'Meaning, and Use of Geomagnetic Indices',
!      Geophysical Monograph 22, American Geophysical Union, Washington, D.!.,
!      1980.
!
!  SPECIAL CONSTANTS
!     NONE
!
      INTEGER IKP3
      REAL KP,KPTOAP
      REAL KP3
      REAL KAP(0:27)
!
      DATA KAP/0.,2.,3.,4.,5.,6.,7.,9.,12.,15.,18.,22.,27.,32.,39.,48., &
              56.,67.,80.,94.,111.,132.,154.,179.,207.,236.,300.,400./
!
      IF(KP .LT. 0.) THEN
         KP3=0.
      ELSE IF(KP .GT. 9.) THEN
         KP3=27.
      ELSE
         KP3=KP*3.
      ENDIF
      IKP3=MIN(INT(KP3),26)
      KPTOAP=KAP(IKP3)+(KAP(IKP3+1)-KAP(IKP3))*(KP3-FLOAT(IKP3))
!
      RETURN
      END
!


      SUBROUTINE PREGION(CGMLT,XLAT,XMLT0,RADIUS,REGB)
!
!  PURPOSE
!     To determine the latitude of the equatorward boundary, maximum
!  and poleward boundary at a given local time given the latitude and
!  local time corresponding to the center of the circular boundary and
!  the radius of this circle.
!
!  METHOD
!    Use the Pythagorean theorem...
!     2  2                       2
!    R +R  - 2.*R R *Cos(Phi) = R
!     0  1       0 1
!
!   Where R = The colatitude of the circle center
!          0
!         R = The circle radius in degrees
!
!       Phi = The current local time - local time of the circle center,
!             in degrees
!
!        R = The colatitude of the boundary at the current local time
!         1
!
!        2                          2   2
!       R  - R *(2.*R *Cos(phi)) + R - R = 0
!        1    1      0              0
! Or
!                               2    2        2   2
!      R = R *Cos(Phi) +- SQRT{R *Cos (Phi) +R - R }
!       1   0                   0                 0
!
!     Choose the postive solution ...in the case of two positive
!   solutions, The smaller one is chosen.
!
!  INPUT PARAMETERS
!    NAME    TYPE       ARRAY  description
!    CGMLT   REAL              The current local time
!    RAD0    REAL       (3)    The radius of the circular boundary
!                              1) Equatorward value
!                              2) Value of maximum
!                              3) Poleward value
!    XLAT    REAL       (3)    The latitude of the center of the circular
!                              boundary
!                              1) Equatorward value
!                              2) Value of maximum
!                              3) Poleward value
!    XMLT0   REAL       (3)    The local time of the center of the circular
!                              boundary
!                              1) Equatorward value
!                              2) Value of maximum
!                              3) Poleward value
!
!  OUTPUT PARAMETERS
!    NAME    TYPE       ARRAY  description
!    REGB    REAL       (3)    The latitude of the circular boundary at local
!                              time CGMLT
!                              1) Equatorward value
!                              2) Value of maximum
!                              3) Poleward value
!
!
!  LOCAL VARIABLES
!    NAME    TYPE       ARRAY  description
!    J       INTEGER           Do loop index
!    SOL1    REAL              One solution of the quadratic equation
!    SOL2    REAL              One solution of the quadratic equation
!    TH1     REAL              The difference between the current local
!                              time and the local time of the center of
!                              the circular  boundary, in degrees, = Phi
!                              in description above.
!    XLT0    REAL              The colatitude of the center of the circ-
!                              ular boundary, in degrees, = R  in description
!                              above.                        0
!
!  SUBROUTINES CALLED
!     NONE
!
!  FUNCTIONS CALLED
!     NONE
!
!  FILES ACCESSED
!     NONE
!
!  AUTHOR
!     William G. Whartenby
!     Computational Physics, Inc.
!     240 Bear Hill Road  Suite 202A
!     Waltham, MA 02154  USA
!     (617)-487-2250
!
!  VERSION
!     1.0.6   01-Dec-1990
!
!  MODIFICATIONS
!     ----Person----  ----Date----  ----------------Description----------------
!
!  REFERENCES
!     NONE
!
!  SPECIAL CONSTANTS
!     EPS = -1.E-3... The maximum a discriminant may go negative
!     and be considered valid (Set equal to 0.0)
!
      REAL EPS
      PARAMETER (EPS = -1.E-3)
      REAL XLAT(3),XMLT0(3),RADIUS(3),REGB(3),DISC,XLT0,TH1,XCT1
      REAL SOL1,SOL2,CGMLT
      INTEGER J
!
!
      DO J = 1,3
!      Compute the discriminant from the equation described above...
!
       XLT0 = 90.-XLAT(J)
       TH1 = (CGMLT-XMLT0(J))*3.1415926/12.
       XCT1 = XLT0*COS(TH1)
       DISC = XCT1**2 - (XLT0**2-RADIUS(J)**2)
!
       IF ( DISC .LT. EPS) THEN
!
!       There is an error
        PRINT *,' Index, lat,local time radius,discriminant = '
        PRINT *,CGMLT,J,XLAT(J),XMLT0(J),RADIUS(J),DISC
        STOP 'NEGATIVE DISCRIMINANT IN PREGION'
!
       ELSE
!
!       Compute the two solutions
!
        IF ( DISC .LT. 0.0) DISC = 0.0
        SOL1 = XCT1 +SQRT(DISC)
        SOL2 = XCT1 -SQRT(DISC)
        IF (MIN(SOL1,SOL2) .GT. 0) THEN
!        Take the minimum
         REGB(J)=MIN(SOL1,SOL2)
!
!        Otherwise , take the postitve one
        ELSE IF (SOL1 .GT. 0) THEN
         REGB(J)=SOL1
        ELSE IF (SOL1 .GT. 0) THEN
         REGB(J)=SOL2
        ELSE
         PRINT *,' Index, lat,local time radius = '
         PRINT *,CGMLT,J,XLAT(J),XMLT0(J),RADIUS(J)
         STOP 'NEGATIVE SOLUTION IN PREGION'
        ENDIF
!       Return the latitude
!
        REGB(J) = 90.-REGB(J)
       ENDIF
      ENDDO
      RETURN
      END
